'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var popmotion = require('popmotion');
var heyListen = require('hey-listen');
var styleValueTypes = require('style-value-types');
var styler = require('stylefire');
var styler__default = _interopDefault(styler);
var sync = require('framesync');
var sync__default = _interopDefault(sync);
var isPropValid = _interopDefault(require('@emotion/is-prop-valid'));
var React = require('react');
var popcorn = require('@popmotion/popcorn');

var htmlTuple = function (args) { return args; };
var svgTuple = function (args) { return args; };
var htmlElements = htmlTuple([
    "a",
    "abbr",
    "address",
    "area",
    "article",
    "aside",
    "audio",
    "b",
    "base",
    "bdi",
    "bdo",
    "big",
    "blockquote",
    "body",
    "br",
    "button",
    "canvas",
    "caption",
    "cite",
    "code",
    "col",
    "colgroup",
    "data",
    "datalist",
    "dd",
    "del",
    "details",
    "dfn",
    "dialog",
    "div",
    "dl",
    "dt",
    "em",
    "embed",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "head",
    "header",
    "hgroup",
    "hr",
    "html",
    "i",
    "iframe",
    "img",
    "input",
    "ins",
    "kbd",
    "keygen",
    "label",
    "legend",
    "li",
    "link",
    "main",
    "map",
    "mark",
    "menu",
    "menuitem",
    "meta",
    "meter",
    "nav",
    "noscript",
    "object",
    "ol",
    "optgroup",
    "option",
    "output",
    "p",
    "param",
    "picture",
    "pre",
    "progress",
    "q",
    "rp",
    "rt",
    "ruby",
    "s",
    "samp",
    "script",
    "section",
    "select",
    "small",
    "source",
    "span",
    "strong",
    "style",
    "sub",
    "summary",
    "sup",
    "table",
    "tbody",
    "td",
    "textarea",
    "tfoot",
    "th",
    "thead",
    "time",
    "title",
    "tr",
    "track",
    "u",
    "ul",
    "var",
    "video",
    "wbr",
    "webview",
]);
var svgElements = svgTuple([
    "animate",
    "circle",
    "clipPath",
    "defs",
    "desc",
    "ellipse",
    "feBlend",
    "feColorMatrix",
    "feComponentTransfer",
    "feComposite",
    "feConvolveMatrix",
    "feDiffuseLighting",
    "feDisplacementMap",
    "feDistantLight",
    "feDropShadow",
    "feFlood",
    "feFuncA",
    "feFuncB",
    "feFuncG",
    "feFuncR",
    "feGaussianBlur",
    "feImage",
    "feMerge",
    "feMergeNode",
    "feMorphology",
    "feOffset",
    "fePointLight",
    "feSpecularLighting",
    "feSpotLight",
    "feTile",
    "feTurbulence",
    "filter",
    "foreignObject",
    "g",
    "image",
    "line",
    "linearGradient",
    "marker",
    "mask",
    "metadata",
    "path",
    "pattern",
    "polygon",
    "polyline",
    "radialGradient",
    "rect",
    "stop",
    "svg",
    "switch",
    "symbol",
    "text",
    "textPath",
    "tspan",
    "use",
    "view",
]);
var elements = htmlElements.concat(svgElements);

var MotionContext = React.createContext({
    dragging: false,
});
var isTarget = function (v) {
    return v !== undefined && typeof v !== "string" && !Array.isArray(v);
};
var useMotionContext = function (controls, initial) {
    var parentContext = React.useContext(MotionContext);
    var context = React.useMemo(function () { return ({
        controls: controls,
        initial: initial && !isTarget(initial) ? initial : parentContext.initial,
    }); }, []);
    context.dragging = parentContext.dragging;
    React.useMemo(function () {
        var initialToApply = initial || parentContext.initial;
        initialToApply && controls.apply(initialToApply);
    }, []);
    return context;
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

var isFloat = function (value) {
    return !isNaN(parseFloat(value));
};
var MotionValue = (function () {
    function MotionValue(init, _a) {
        var _b = _a === void 0 ? {} : _a, transformer = _b.transformer, parent = _b.parent;
        var _this = this;
        this.timeDelta = 0;
        this.lastUpdated = 0;
        this.canTrackVelocity = false;
        this.notifySubscriber = function (subscriber) {
            subscriber(_this.current);
        };
        this.scheduleVelocityCheck = function () { return sync__default.postRender(_this.velocityCheck); };
        this.velocityCheck = function (_a) {
            var timestamp = _a.timestamp;
            if (timestamp !== _this.lastUpdated) {
                _this.prev = _this.current;
            }
        };
        this.setChild = function (child) { return child.set(_this.current); };
        this.parent = parent;
        this.transformer = transformer;
        this.set(init);
        this.canTrackVelocity = isFloat(this.current);
    }
    MotionValue.prototype.addChild = function (config) {
        var child = new MotionValue(this.current, __assign({ parent: this }, config));
        if (!this.children)
            this.children = new Set();
        this.children.add(child);
        return child;
    };
    MotionValue.prototype.removeChild = function (child) {
        if (!this.children) {
            return;
        }
        this.children.delete(child);
    };
    MotionValue.prototype.subscribeTo = function (subscriptions, subscription) {
        var _this = this;
        var updateSubscriber = function () { return subscription(_this.current); };
        subscriptions.add(updateSubscriber);
        return function () { return subscriptions.delete(updateSubscriber); };
    };
    MotionValue.prototype.addUpdateSubscription = function (subscription) {
        if (!this.updateSubscribers)
            this.updateSubscribers = new Set();
        return this.subscribeTo(this.updateSubscribers, subscription);
    };
    MotionValue.prototype.addRenderSubscription = function (subscription) {
        if (!this.renderSubscribers)
            this.renderSubscribers = new Set();
        return this.subscribeTo(this.renderSubscribers, subscription);
    };
    MotionValue.prototype.set = function (v, render) {
        if (render === void 0) { render = true; }
        this.prev = this.current;
        this.current = this.transformer ? this.transformer(v) : v;
        if (this.updateSubscribers) {
            this.updateSubscribers.forEach(this.notifySubscriber);
        }
        if (this.children) {
            this.children.forEach(this.setChild);
        }
        if (render && this.renderSubscribers) {
            this.renderSubscribers.forEach(this.notifySubscriber);
        }
        var _a = sync.getFrameData(), delta = _a.delta, timestamp = _a.timestamp;
        if (this.lastUpdated !== timestamp) {
            this.timeDelta = delta;
            this.lastUpdated = timestamp;
            sync__default.postRender(this.scheduleVelocityCheck);
        }
    };
    MotionValue.prototype.get = function () {
        return this.current;
    };
    MotionValue.prototype.getVelocity = function () {
        return this.canTrackVelocity
            ?
                popcorn.velocityPerSecond(parseFloat(this.current) -
                    parseFloat(this.prev), this.timeDelta)
            : 0;
    };
    MotionValue.prototype.control = function (controller, _a, transformer) {
        var _this = this;
        var delay = _a.delay, config = __rest(_a, ["delay"]);
        this.stop();
        var initialisedController = controller(__assign({ from: this.get(), velocity: this.getVelocity() }, config));
        if (transformer) {
            initialisedController = initialisedController.pipe(transformer);
        }
        if (delay) {
            initialisedController = popmotion.chain(popmotion.delay(delay), initialisedController);
        }
        return new Promise(function (complete) {
            _this.controller = initialisedController.start({
                update: function (v) { return _this.set(v); },
                complete: complete,
            });
        });
    };
    MotionValue.prototype.stop = function () {
        if (this.controller)
            this.controller.stop();
    };
    MotionValue.prototype.destroy = function () {
        this.updateSubscribers && this.updateSubscribers.clear();
        this.renderSubscribers && this.renderSubscribers.clear();
        this.parent && this.parent.removeChild(this);
        this.stop();
    };
    return MotionValue;
}());
var motionValue = function (init, opts) {
    return new MotionValue(init, opts);
};

var updateStyler = styler.createStylerFactory({
    onRead: function () { return null; },
    onRender: function (state, _a) {
        var onUpdate = _a.onUpdate;
        return onUpdate(state);
    },
});
var MotionValuesMap = (function () {
    function MotionValuesMap() {
        this.hasMounted = false;
        this.shouldRender = true;
        this.values = new Map();
        this.unsubscribers = new Map();
    }
    MotionValuesMap.prototype.has = function (key) {
        return this.values.has(key);
    };
    MotionValuesMap.prototype.set = function (key, value) {
        this.values.set(key, value);
        if (this.hasMounted)
            this.bindValueToStyler(key, value);
    };
    MotionValuesMap.prototype.get = function (key, defaultValue) {
        var value = this.values.get(key);
        if (value === undefined && defaultValue !== undefined) {
            value = new MotionValue(defaultValue);
            this.set(key, value);
        }
        return value;
    };
    MotionValuesMap.prototype.forEach = function (callback) {
        return this.values.forEach(callback);
    };
    MotionValuesMap.prototype.bindValueToStyler = function (key, value) {
        var _this = this;
        var update = function (v) {
            if (_this.shouldRender) {
                _this.styler.set(key, v);
            }
            if (_this.onUpdate) {
                _this.onUpdate.set(key, v);
            }
        };
        var unsubscribe = value.addRenderSubscription(update);
        this.unsubscribers.set(key, unsubscribe);
    };
    MotionValuesMap.prototype.setOnUpdate = function (onUpdate) {
        this.onUpdate = undefined;
        if (onUpdate) {
            this.onUpdate = updateStyler({ onUpdate: onUpdate });
        }
    };
    MotionValuesMap.prototype.setShouldRender = function (render) {
        this.shouldRender = render;
    };
    MotionValuesMap.prototype.mount = function (element) {
        var _this = this;
        this.hasMounted = true;
        this.styler = styler__default(element, { preparseOutput: false });
        this.values.forEach(function (value, key) { return _this.bindValueToStyler(key, value); });
    };
    MotionValuesMap.prototype.unmount = function () {
        var _this = this;
        this.values.forEach(function (_value, key) {
            var unsubscribe = _this.unsubscribers.get(key);
            unsubscribe && unsubscribe();
        });
    };
    return MotionValuesMap;
}());
var useMotionValues = function (ref, _a) {
    var onUpdate = _a.onUpdate, _b = _a.render, render = _b === void 0 ? true : _b;
    var motionValues = React.useMemo(function () { return new MotionValuesMap(); }, []);
    motionValues.setOnUpdate(onUpdate);
    motionValues.setShouldRender(render);
    React.useEffect(function () {
        heyListen.invariant(ref.current instanceof Element, "No `ref` found. Ensure components created with `motion.custom` forward refs using `React.forwardRef`");
        motionValues.mount(ref.current);
        return function () { return motionValues.unmount(); };
    });
    return motionValues;
};

var underDampedSpring = function () { return ({
    type: "spring",
    stiffness: 500,
    damping: 25,
    restDelta: 0.5,
    restSpeed: 10,
}); };
var overDampedSpring = function (to) { return ({
    type: "spring",
    stiffness: 700,
    damping: to === 0 ? 100 : 35,
}); };
var linearTween = function () { return ({
    ease: "linear",
    duration: 250,
}); };
var defaultTransitions = {
    x: underDampedSpring,
    y: underDampedSpring,
    z: underDampedSpring,
    rotate: underDampedSpring,
    rotateX: underDampedSpring,
    rotateY: underDampedSpring,
    rotateZ: underDampedSpring,
    scaleX: overDampedSpring,
    scaleY: overDampedSpring,
    scale: overDampedSpring,
    opacity: linearTween,
    backgroundColor: linearTween,
    color: linearTween,
    default: overDampedSpring,
};
var getDefaultTransition = function (valueKey, to) {
    var transitionFactory = defaultTransitions[valueKey] || defaultTransitions.default;
    return __assign({}, transitionFactory(to), { to: to });
};

var just = function (_a) {
    var to = _a.to;
    return popmotion.action(function (_a) {
        var update = _a.update, complete = _a.complete;
        update(to);
        complete();
    });
};
var transitions = { tween: popmotion.tween, spring: popmotion.spring, keyframes: popmotion.keyframes, decay: popmotion.decay, physics: popmotion.physics, inertia: popmotion.inertia, just: just };
var linear = popmotion.easing.linear, easeIn = popmotion.easing.easeIn, easeOut = popmotion.easing.easeOut, easeInOut = popmotion.easing.easeInOut, circIn = popmotion.easing.circIn, circOut = popmotion.easing.circOut, circInOut = popmotion.easing.circInOut, backIn = popmotion.easing.backIn, backOut = popmotion.easing.backOut, backInOut = popmotion.easing.backInOut, anticipate = popmotion.easing.anticipate;
var easingLookup = {
    linear: linear,
    easeIn: easeIn,
    easeOut: easeOut,
    easeInOut: easeInOut,
    circIn: circIn,
    circOut: circOut,
    circInOut: circInOut,
    backIn: backIn,
    backOut: backOut,
    backInOut: backInOut,
    anticipate: anticipate,
};
var transitionOptionParser = {
    tween: function (opts) {
        var ease = opts.ease;
        if (Array.isArray(ease)) {
            heyListen.invariant(ease.length === 4, "Cubic bezier arrays must contain four numerical values.");
            var x1 = ease[0], y1 = ease[1], x2 = ease[2], y2 = ease[3];
            opts.ease = popmotion.easing.cubicBezier(x1, y1, x2, y2);
        }
        else if (typeof ease === "string") {
            heyListen.invariant(easingLookup[ease] !== undefined, "Invalid easing type '" + ease + "'");
            opts.ease = easingLookup[ease];
        }
        return opts;
    },
    keyframes: function (_a) {
        var from = _a.from, to = _a.to, opts = __rest(_a, ["from", "to"]);
        return opts;
    },
};
var isTransitionDefined = function (_a) {
    var beforeChildren = _a.beforeChildren, afterChildren = _a.afterChildren, delay = _a.delay, delayChildren = _a.delayChildren, staggerChildren = _a.staggerChildren, staggerDirection = _a.staggerDirection, transition = __rest(_a, ["beforeChildren", "afterChildren", "delay", "delayChildren", "staggerChildren", "staggerDirection"]);
    return Object.keys(transition).length;
};
var getTransitionForValue = function (key, to, transitionDefinition) {
    var delay = transitionDefinition ? transitionDefinition.delay : 0;
    if (transitionDefinition === undefined ||
        !isTransitionDefined(transitionDefinition)) {
        return __assign({ delay: delay }, getDefaultTransition(key, to));
    }
    var valueTransitionDefinition = transitionDefinition[key] ||
        transitionDefinition.default ||
        transitionDefinition;
    return valueTransitionDefinition.type === false
        ? { type: "just", delay: delay, to: to }
        : __assign({ delay: delay,
            to: to }, valueTransitionDefinition);
};
var preprocessOptions = function (type, opts) {
    return transitionOptionParser[type] ? transitionOptionParser[type](opts) : opts;
};
var getTransition = function (valueKey, to, transition) {
    var _a = getTransitionForValue(valueKey, to, transition), _b = _a.type, type = _b === void 0 ? "tween" : _b, transitionDefinition = __rest(_a, ["type"]);
    var actionFactory = transitions[type];
    var opts = preprocessOptions(type, transitionDefinition);
    return [actionFactory, opts];
};

var auto = {
    test: function (v) { return v === "auto"; },
    parse: function (v) { return v; },
};
var valueTypeTests = [styleValueTypes.number, styleValueTypes.degrees, styleValueTypes.percent, styleValueTypes.px, styleValueTypes.vw, styleValueTypes.vh, auto];
var testValueType = function (v) { return function (type) { return type.test(v); }; };
var getValueType = function (v) { return valueTypeTests.find(testValueType(v)); };

var positionalKeys = new Set([
    "width",
    "height",
    "top",
    "left",
    "right",
    "bottom",
    "x",
    "y",
]);
var isPositionalKey = function (key) { return positionalKeys.has(key); };
var hasPositionalKey = function (target) {
    return Object.keys(target).some(isPositionalKey);
};
var setAndResetVelocity = function (value, to) {
    value.set(to, false);
    value.set(to);
};
var BoundingBoxDimension;
(function (BoundingBoxDimension) {
    BoundingBoxDimension["width"] = "width";
    BoundingBoxDimension["height"] = "height";
    BoundingBoxDimension["left"] = "left";
    BoundingBoxDimension["right"] = "right";
    BoundingBoxDimension["top"] = "top";
    BoundingBoxDimension["bottom"] = "bottom";
})(BoundingBoxDimension || (BoundingBoxDimension = {}));
var getPosFromMatrix = function (matrix, pos) {
    return parseFloat(matrix.split(", ")[pos]);
};
var getTranslateFromMatrix = function (pos2, pos3) { return function (_bbox, _a) {
    var transform = _a.transform;
    if (transform === "none" || !transform)
        return 0;
    var matrix3d = transform.match(/^matrix3d\((.+)\)$/);
    if (matrix3d) {
        return getPosFromMatrix(matrix3d[1], pos3);
    }
    else {
        var matrix = transform.match(/^matrix\((.+)\)$/);
        return getPosFromMatrix(matrix[1], pos2);
    }
}; };
var positionalValues = {
    width: function (_a) {
        var width = _a.width;
        return width;
    },
    height: function (_a) {
        var height = _a.height;
        return height;
    },
    top: function (_bbox, _a) {
        var top = _a.top;
        return parseFloat(top);
    },
    left: function (_bbox, _a) {
        var left = _a.left;
        return parseFloat(left);
    },
    bottom: function (_a, _b) {
        var height = _a.height;
        var top = _b.top;
        return parseFloat(top) + height;
    },
    right: function (_a, _b) {
        var width = _a.width;
        var left = _b.left;
        return parseFloat(left) + width;
    },
    x: getTranslateFromMatrix(4, 13),
    y: getTranslateFromMatrix(5, 14),
};
var convertChangedValueTypes = function (pose, values, ref, changedKeys) {
    var element = ref.current;
    var elementStyler = styler__default(element);
    var originBbox = element.getBoundingClientRect();
    var elementComputedStyle = getComputedStyle(element);
    var top = elementComputedStyle.top, left = elementComputedStyle.left, bottom = elementComputedStyle.bottom, right = elementComputedStyle.right, transform = elementComputedStyle.transform;
    var originComputedStyle = { top: top, left: left, bottom: bottom, right: right, transform: transform };
    elementStyler.render();
    var targetBbox = element.getBoundingClientRect();
    changedKeys.forEach(function (key) {
        var value = values.get(key);
        setAndResetVelocity(value, positionalValues[key](originBbox, originComputedStyle));
        pose[key] = positionalValues[key](targetBbox, elementComputedStyle);
    });
    elementStyler.render();
    return pose;
};
var checkAndConvertChangedValueTypes = function (values, ref, target, transitionEnd) {
    if (transitionEnd === void 0) { transitionEnd = {}; }
    transitionEnd = __assign({}, transitionEnd);
    var posePositionalKeys = Object.keys(target).filter(isPositionalKey);
    var changedValueTypeKeys = posePositionalKeys.reduce(function (acc, key) {
        var value = values.get(key);
        var from = value.get();
        var to = target[key];
        var fromType = getValueType(from);
        var toType = getValueType(to);
        if (fromType !== toType) {
            acc.push(key);
            transitionEnd[key] =
                transitionEnd[key] !== undefined
                    ? transitionEnd[key]
                    : target[key];
            setAndResetVelocity(value, to);
        }
        return acc;
    }, []);
    return changedValueTypeKeys.length
        ? {
            target: convertChangedValueTypes(target, values, ref, changedValueTypeKeys),
            transitionEnd: transitionEnd,
        }
        : { target: target, transitionEnd: transitionEnd };
};
var unitConversion = function (values, ref, target, transitionEnd) {
    return hasPositionalKey(target)
        ? checkAndConvertChangedValueTypes(values, ref, target, transitionEnd)
        : { target: target, transitionEnd: transitionEnd };
};

var getCurrent = function (values) {
    var current = {};
    values.forEach(function (value, key) { return (current[key] = value.get()); });
    return current;
};
var getVelocity = function (values) {
    var velocity = {};
    values.forEach(function (value, key) { return (velocity[key] = value.getVelocity()); });
    return velocity;
};
var isAnimatable = function (value) {
    return typeof value === "number" || styleValueTypes.complex.test(value);
};
var isTargetResolver = function (p) {
    return typeof p === "function";
};
var isVariantLabels = function (v) { return Array.isArray(v); };
var AnimationControls = (function () {
    function AnimationControls(values, ref) {
        var _this = this;
        this.variants = {};
        this.baseTarget = {};
        this.overrides = [];
        this.resolvedOverrides = [];
        this.isAnimating = new Set();
        this.values = values;
        this.ref = ref;
        this.values.forEach(function (value, key) { return (_this.baseTarget[key] = value.get()); });
    }
    AnimationControls.prototype.setProps = function (props) {
        this.props = props;
    };
    AnimationControls.prototype.setVariants = function (variants) {
        if (variants)
            this.variants = variants;
    };
    AnimationControls.prototype.setDefaultTransition = function (transition) {
        if (transition)
            this.defaultTransition = transition;
    };
    AnimationControls.prototype.setValues = function (target, isActive) {
        var _this = this;
        if (isActive === void 0) { isActive = new Set(); }
        return Object.keys(target).forEach(function (key) {
            if (isActive.has(key))
                return;
            isActive.add(key);
            if (_this.values.has(key)) {
                var value = _this.values.get(key);
                value && value.set(target[key]);
            }
            else {
                _this.values.set(key, motionValue(target[key]));
            }
            _this.baseTarget[key] = target[key];
        });
    };
    AnimationControls.prototype.checkForNewValues = function (target) {
        var _this = this;
        var newValueKeys = Object.keys(target).filter(function (key) { return !_this.values.has(key); });
        if (!newValueKeys.length)
            return;
        var domStyler = styler__default(this.ref.current);
        newValueKeys.forEach(function (key) {
            var domValue = domStyler.get(key);
            _this.values.set(key, motionValue(domValue));
            _this.baseTarget[key] = domValue;
        });
    };
    AnimationControls.prototype.resolveVariant = function (variant) {
        if (!variant) {
            return {
                target: undefined,
                transition: undefined,
                transitionEnd: undefined,
            };
        }
        if (isTargetResolver(variant)) {
            variant = variant(this.props, getCurrent(this.values), getVelocity(this.values));
        }
        var transition = variant.transition, transitionEnd = variant.transitionEnd, target = __rest(variant, ["transition", "transitionEnd"]);
        return { transition: transition, transitionEnd: transitionEnd, target: target };
    };
    AnimationControls.prototype.getHighestPriority = function () {
        var highest = 0;
        var numOverrides = this.overrides.length;
        if (!numOverrides)
            return highest;
        for (var i = 0; i < numOverrides; i++) {
            if (this.overrides[i] !== undefined) {
                highest = i;
            }
        }
        return highest;
    };
    AnimationControls.prototype.clearOverride = function (overrideIndex) {
        var _this = this;
        var override = this.overrides[overrideIndex];
        if (!override)
            return;
        this.overrides[overrideIndex] = undefined;
        var highest = this.getHighestPriority();
        this.resetIsAnimating();
        if (highest >= overrideIndex) {
            var highestOverride = this.overrides[highest];
            highestOverride &&
                this.start(highestOverride, { priority: highest });
        }
        var overrideTarget = this.resolvedOverrides[overrideIndex];
        if (!overrideTarget)
            return;
        var remainingValues = Object.keys(this.baseTarget).reduce(function (acc, key) {
            if (overrideTarget[key] !== undefined) {
                acc[key] = _this.baseTarget[key];
            }
            return acc;
        }, {});
        this.animate(remainingValues);
    };
    AnimationControls.prototype.apply = function (definition) {
        if (Array.isArray(definition)) {
            return this.applyVariantLabels(definition);
        }
        else if (typeof definition === "string") {
            return this.applyVariantLabels([definition]);
        }
        else {
            this.setValues(definition);
        }
    };
    AnimationControls.prototype.applyVariantLabels = function (variantLabelList) {
        var _this = this;
        var isSetting = new Set();
        var reversedList = variantLabelList.slice().reverse();
        reversedList.forEach(function (key) {
            var _a = _this.resolveVariant(_this.variants[key]), target = _a.target, transitionEnd = _a.transitionEnd;
            if (transitionEnd) {
                _this.setValues(transitionEnd, isSetting);
            }
            if (target) {
                _this.setValues(target, isSetting);
            }
        });
    };
    AnimationControls.prototype.start = function (definition, opts) {
        if (opts === void 0) { opts = {}; }
        this.setPriorityAnimation(definition, opts.priority || 0);
        this.resetIsAnimating(opts.priority);
        if (isVariantLabels(definition)) {
            return this.animateVariantLabels(definition, opts);
        }
        else if (typeof definition === "string") {
            return this.animateVariant(definition, opts);
        }
        else {
            return this.animate(definition, opts);
        }
    };
    AnimationControls.prototype.setPriorityAnimation = function (definition, priority) {
        this.overrides[priority] = definition;
    };
    AnimationControls.prototype.isHighestPriority = function (priority) {
        var numOverrides = this.overrides.length;
        for (var i = priority + 1; i < numOverrides; i++) {
            if (this.overrides[i] !== undefined) {
                return false;
            }
        }
        return true;
    };
    AnimationControls.prototype.animate = function (animationDefinition, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.priority, priority = _d === void 0 ? 0 : _d;
        var _e = this.resolveVariant(animationDefinition), target = _e.target, transition = _e.transition, transitionEnd = _e.transitionEnd;
        if (!target)
            return Promise.resolve();
        if (priority) {
            this.resolvedOverrides[priority] = target;
        }
        this.checkForNewValues(target);
        var converted = unitConversion(this.values, this.ref, target, transitionEnd);
        target = converted.target;
        transitionEnd = converted.transitionEnd;
        if (!transition && this.defaultTransition) {
            transition = this.defaultTransition;
        }
        var animations = Object.keys(target).reduce(function (acc, key) {
            var value = _this.values.get(key);
            if (_this.isAnimating.has(key) ||
                !target ||
                !value ||
                target[key] === undefined) {
                return acc;
            }
            var valueTarget = target[key];
            if (!priority) {
                _this.baseTarget[key] = valueTarget;
            }
            if (isAnimatable(valueTarget)) {
                var _a = getTransition(key, valueTarget, __assign({ delay: delay }, transition)), action = _a[0], options = _a[1];
                acc.push(value.control(action, options));
            }
            else {
                value.set(valueTarget);
            }
            _this.isAnimating.add(key);
            return acc;
        }, []);
        return Promise.all(animations).then(function () {
            if (!transitionEnd)
                return;
            _this.setValues(transitionEnd);
        });
    };
    AnimationControls.prototype.animateVariantLabels = function (variantLabels, opts) {
        var _this = this;
        var animations = variantLabels.slice().reverse()
            .map(function (label) { return _this.animateVariant(label, opts); });
        return Promise.all(animations);
    };
    AnimationControls.prototype.animateVariant = function (variantLabel, opts) {
        var _this = this;
        var beforeChildren = false;
        var afterChildren = false;
        var delayChildren = 0;
        var staggerChildren = 0;
        var staggerDirection = 1;
        var priority = (opts && opts.priority) || 0;
        var variant = this.variants[variantLabel];
        var getAnimations = variant
            ? function () { return _this.animate(variant, opts); }
            : function () { return Promise.resolve(); };
        var getChildrenAnimations = this.children
            ? function () {
                return _this.animateChildren(variantLabel, delayChildren, staggerChildren, staggerDirection, priority);
            }
            : function () { return Promise.resolve(); };
        if (variant && this.children) {
            var transition = this.resolveVariant(variant).transition;
            if (transition) {
                beforeChildren = transition.beforeChildren || beforeChildren;
                afterChildren = transition.afterChildren || afterChildren;
                delayChildren = transition.delayChildren || delayChildren;
                staggerChildren = transition.staggerChildren || staggerChildren;
                staggerDirection =
                    transition.staggerDirection || staggerDirection;
            }
        }
        if (beforeChildren || afterChildren) {
            var _a = beforeChildren
                ? [getAnimations, getChildrenAnimations]
                : [getChildrenAnimations, getAnimations], first = _a[0], last = _a[1];
            return first().then(last);
        }
        else {
            return Promise.all([getAnimations(), getChildrenAnimations()]);
        }
    };
    AnimationControls.prototype.animateChildren = function (variantLabel, delayChildren, staggerChildren, staggerDirection, priority) {
        if (delayChildren === void 0) { delayChildren = 0; }
        if (staggerChildren === void 0) { staggerChildren = 0; }
        if (staggerDirection === void 0) { staggerDirection = 1; }
        if (priority === void 0) { priority = 0; }
        if (!this.children) {
            return Promise.resolve();
        }
        var animations = [];
        var maxStaggerDuration = (this.children.size - 1) * staggerChildren;
        var generateStaggerDuration = staggerDirection === 1
            ? function (i) { return i * staggerChildren; }
            : function (i) { return maxStaggerDuration - i * staggerChildren; };
        Array.from(this.children).forEach(function (childControls, i) {
            var animation = childControls.animateVariant(variantLabel, {
                priority: priority,
                delay: delayChildren + generateStaggerDuration(i),
            });
            animations.push(animation);
        });
        return Promise.all(animations);
    };
    AnimationControls.prototype.checkOverrideIsAnimating = function (priority) {
        var numOverrides = this.overrides.length;
        for (var i = priority + 1; i < numOverrides; i++) {
            var resolvedOverride = this.resolvedOverrides[i];
            if (resolvedOverride) {
                for (var key in resolvedOverride) {
                    this.isAnimating.add(key);
                }
            }
        }
    };
    AnimationControls.prototype.resetIsAnimating = function (priority) {
        if (priority === void 0) { priority = 0; }
        this.isAnimating.clear();
        if (priority < this.getHighestPriority()) {
            this.checkOverrideIsAnimating(priority);
        }
        if (this.children) {
            this.children.forEach(function (child) { return child.resetIsAnimating(priority); });
        }
    };
    AnimationControls.prototype.stop = function () {
        this.values.forEach(function (value) { return value.stop(); });
    };
    AnimationControls.prototype.addChild = function (controls) {
        if (!this.children) {
            this.children = new Set();
        }
        this.children.add(controls);
    };
    AnimationControls.prototype.removeChild = function (controls) {
        if (!this.children) {
            return;
        }
        this.children.delete(controls);
    };
    AnimationControls.prototype.resetChildren = function () {
        if (this.children)
            this.children.clear();
    };
    return AnimationControls;
}());
var useAnimationControls = function (values, props, ref, inherit) {
    var variants = props.variants, transition = props.transition;
    var parentControls = React.useContext(MotionContext).controls;
    var controls = React.useMemo(function () { return new AnimationControls(values, ref); }, []);
    controls.resetChildren();
    if (inherit && parentControls) {
        parentControls.addChild(controls);
    }
    React.useEffect(function () { return function () { return parentControls && parentControls.removeChild(controls); }; }, []);
    controls.setProps(props);
    controls.setVariants(variants);
    controls.setDefaultTransition(transition);
    return controls;
};

var useExternalRef = function (external) {
    var ref = !external || typeof external === "function" ? React.useRef(null) : external;
    React.useEffect(function () {
        if (external && typeof external === "function") {
            external(ref.current);
            return function () { return external(null); };
        }
    });
    return ref;
};

var createValueResolver = function (resolver) { return function (values) {
    var resolvedValues = {};
    values.forEach(function (value, key) { return (resolvedValues[key] = resolver(value)); });
    return resolvedValues;
}; };
var resolveCurrent = createValueResolver(function (value) { return value.get(); });

var isMotionValue = function (value) { return value instanceof MotionValue; };
var buildStyleAttr = function (values, styleProp) {
    return __assign({}, styleProp, styler.buildStyleProperty(resolveCurrent(values)));
};
var addMotionStyles = function (values, styleProp) {
    if (styleProp === void 0) { styleProp = {}; }
    var style = {};
    for (var key in styleProp) {
        var thisStyle = styleProp[key];
        if (isMotionValue(thisStyle)) {
            values.set(key, thisStyle);
        }
        else if (styler.isTransformProp(key)) {
            if (!values.has(key)) {
                values.set(key, motionValue(thisStyle));
            }
        }
        else {
            style[key] = thisStyle;
        }
    }
    return style;
};

var AnimationManager = (function () {
    function AnimationManager() {
        this.hasMounted = false;
        this.pendingAnimations = [];
        this.componentControls = new Set();
        this.variants = {};
    }
    AnimationManager.prototype.setVariants = function (variants) {
        this.variants = variants;
        this.componentControls.forEach(function (controls) {
            return controls.setVariants(variants);
        });
    };
    AnimationManager.prototype.setDefaultTransition = function (transition) {
        this.defaultTransition = transition;
        this.componentControls.forEach(function (controls) {
            return controls.setDefaultTransition(transition);
        });
    };
    AnimationManager.prototype.subscribe = function (controls) {
        var _this = this;
        this.componentControls.add(controls);
        if (this.variants)
            controls.setVariants(this.variants);
        if (this.defaultTransition)
            controls.setDefaultTransition(this.defaultTransition);
        return function () { return _this.componentControls.delete(controls); };
    };
    AnimationManager.prototype.start = function (definition) {
        if (this.hasMounted) {
            var animations_1 = [];
            this.componentControls.forEach(function (controls) {
                var animation = controls.start(definition);
                animations_1.push(animation);
            });
            return Promise.all(animations_1);
        }
        else {
            this.pendingAnimations.push(definition);
            return Promise.resolve();
        }
    };
    AnimationManager.prototype.stop = function () {
        this.componentControls.forEach(function (controls) { return controls.stop(); });
    };
    AnimationManager.prototype.mount = function () {
        var _this = this;
        this.hasMounted = true;
        this.pendingAnimations.forEach(function (variant) { return _this.start(variant); });
    };
    AnimationManager.prototype.unmount = function () {
        this.hasMounted = false;
        this.stop();
    };
    return AnimationManager;
}());

var useAnimationSubscription = function (animation, controls) {
    var unsubscribe = React.useMemo(function () { return animation.subscribe(controls); }, [animation]);
    React.useEffect(function () { return function () { return unsubscribe && unsubscribe(); }; }, [unsubscribe]);
};

var poseToArray = function (pose) {
    if (!pose) {
        return [];
    }
    if (Array.isArray(pose)) {
        return pose;
    }
    return [pose];
};
var resolvePoses = function (pose) {
    var unresolvedPose = pose instanceof MotionValue ? pose.get() : pose;
    return Array.from(new Set(poseToArray(unresolvedPose)));
};
var asDependencyList = function (list) { return [list.join(",")]; };

var hasVariantChanged = function (oldVariant, newVariant) {
    return oldVariant.join(",") !== newVariant.join(",");
};
var useVariants = function (targetVariant, inherit, controls, initialVariant, onAnimationComplete) {
    var variantList = resolvePoses(targetVariant);
    var hasMounted = React.useRef(false);
    React.useEffect(function () {
        if (inherit)
            return;
        if (hasMounted.current || hasVariantChanged(resolvePoses(initialVariant), variantList)) {
            controls.start(variantList).then(function () {
                onAnimationComplete && onAnimationComplete();
            });
        }
        hasMounted.current = true;
    }, asDependencyList(variantList));
};

function isEventTarget(target) {
    return (typeof target.addEventListener === "function" &&
        typeof target.removeEventListener === "function");
}
var eventListener = function (target, name, handler, options) {
    var startListening = function () {
        if (!target) {
            return;
        }
        target.addEventListener(name, handler, options);
    };
    var stopListening = function () {
        if (!target) {
            return;
        }
        target.removeEventListener(name, handler, options);
    };
    return [startListening, stopListening];
};
var useEvent = function (type, target, handler, options) {
    var result = undefined;
    if ((!target || isEventTarget(target)) && handler) {
        result = eventListener(target, type, handler, options);
    }
    React.useEffect(function () {
        var ref = target;
        if (!handler || !target || isEventTarget(target) || !ref.current) {
            return;
        }
        var _a = eventListener(ref.current, type, handler, options), start = _a[0], stop = _a[1];
        start();
        return stop;
    }, [type, target, handler, options]);
    return result;
};

var _TEST_POINTER_DO_NOT_USE = { x: 0, y: 0 };
var pointForTarget = function (_a, target) {
    var _b = _a.pageX, pageX = _b === void 0 ? _TEST_POINTER_DO_NOT_USE.x : _b, _c = _a.pageY, pageY = _c === void 0 ? _TEST_POINTER_DO_NOT_USE.y : _c;
    if (!target) {
        return { x: pageX, y: pageY };
    }
    if (window.webkitConvertPointFromPageToNode) {
        var webkitPoint = new WebKitPoint(pageX, pageY);
        webkitPoint = window.webkitConvertPointFromPageToNode(target, webkitPoint);
        return { x: webkitPoint.x, y: webkitPoint.y };
    }
    var rect = target.getBoundingClientRect();
    var scaleX = 1;
    if (target.style.width && target.style.width !== "") {
        scaleX = parseFloat(target.style.width) / rect.width;
    }
    var scaleY = 1;
    if (target.style.height && target.style.height !== "") {
        scaleY = parseFloat(target.style.height) / rect.height;
    }
    var point = {
        x: scaleX * (pageX - rect.left - target.clientLeft + target.scrollLeft),
        y: scaleY * (pageY - rect.top - target.clientTop + target.scrollTop),
    };
    return point;
};
var extractEventInfo = function (event) {
    var target = event.target instanceof HTMLElement ? event.target : null;
    var point = pointForTarget(event, target);
    var devicePoint = pointForTarget(event, document.body);
    return { point: point, devicePoint: devicePoint };
};
var wrapHandler = function (handler) {
    if (!handler) {
        return undefined;
    }
    var listener = function (event, info) {
        if (!info) {
            info = extractEventInfo(event);
        }
        handler(event, info);
    };
    return listener;
};

var mergeUseEventResults = function () {
    var values = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        values[_i] = arguments[_i];
    }
    if (values.every(function (v) { return v === undefined; })) {
        return;
    }
    var start = function () {
        for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
            var value = values_1[_i];
            if (value) {
                value[0]();
            }
        }
    };
    var stop = function () {
        for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
            var value = values_2[_i];
            if (value) {
                value[1]();
            }
        }
    };
    return [start, stop];
};
var useMouseEvents = function (_a, ref, options) {
    var onMouseDown = _a.onMouseDown, onMouseMove = _a.onMouseMove, onMouseUp = _a.onMouseUp, onMouseOver = _a.onMouseOver, onMouseOut = _a.onMouseOut, onMouseEnter = _a.onMouseEnter, onMouseLeave = _a.onMouseLeave;
    var down = useEvent("mousedown", ref, wrapHandler(onMouseDown), options);
    var move = useEvent("mousemove", ref, wrapHandler(onMouseMove), options);
    var up = useEvent("mouseup", ref, wrapHandler(onMouseUp), options);
    var over = useEvent("mouseover", ref, wrapHandler(onMouseOver), options);
    var out = useEvent("mouseout", ref, wrapHandler(onMouseOut), options);
    var enter = useEvent("mouseenter", ref, wrapHandler(onMouseEnter), options);
    var leave = useEvent("mouseleave", ref, wrapHandler(onMouseLeave), options);
    return mergeUseEventResults(down, move, up, over, out, enter, leave);
};
var useTouchEvents = function (_a, ref, options) {
    var onTouchStart = _a.onTouchStart, onTouchMove = _a.onTouchMove, onTouchEnd = _a.onTouchEnd, onTouchCancel = _a.onTouchCancel;
    var down = useEvent("touchstart", ref, wrapHandler(onTouchStart), options);
    var move = useEvent("touchmove", ref, wrapHandler(onTouchMove), options);
    var up = useEvent("touchend", ref, wrapHandler(onTouchEnd), options);
    var cancel = useEvent("touchcancel", ref, wrapHandler(onTouchCancel), options);
    return mergeUseEventResults(down, move, up, cancel);
};
var useNativePointerEvents = function (_a, ref, options) {
    var onPointerDown = _a.onPointerDown, onPointerMove = _a.onPointerMove, onPointerUp = _a.onPointerUp, onPointerCancel = _a.onPointerCancel, onPointerOver = _a.onPointerOver, onPointerOut = _a.onPointerOut, onPointerEnter = _a.onPointerEnter, onPointerLeave = _a.onPointerLeave;
    var down = useEvent("pointerdown", ref, wrapHandler(onPointerDown), options);
    var move = useEvent("pointermove", ref, wrapHandler(onPointerMove), options);
    var up = useEvent("pointerup", ref, wrapHandler(onPointerUp), options);
    var cancel = useEvent("pointercancel", ref, wrapHandler(onPointerCancel), options);
    var over = useEvent("pointerover", ref, wrapHandler(onPointerOver), options);
    var out = useEvent("pointerout", ref, wrapHandler(onPointerOut), options);
    var enter = useEvent("pointerenter", ref, wrapHandler(onPointerEnter), options);
    var leave = useEvent("pointerleave", ref, wrapHandler(onPointerLeave), options);
    return mergeUseEventResults(down, move, up, cancel, over, out, enter, leave);
};
var supportsPointerEvents = function () {
    return window.onpointerdown === null &&
        window.onpointermove === null &&
        window.onpointerup === null;
};
var supportsTouchEvents = function () {
    return window.ontouchstart === null &&
        window.ontouchmove === null &&
        window.ontouchend === null;
};
var supportsMouseEvents = function () {
    return window.onmousedown === null &&
        window.onmousemove === null &&
        window.onmouseup === null;
};
var usePointerEvents = function (eventHandlers, ref, options) {
    var onPointerDown = eventHandlers.onPointerDown, onPointerMove = eventHandlers.onPointerMove, onPointerUp = eventHandlers.onPointerUp, onPointerCancel = eventHandlers.onPointerCancel, onPointerOver = eventHandlers.onPointerOver, onPointerOut = eventHandlers.onPointerOut, onPointerEnter = eventHandlers.onPointerEnter, onPointerLeave = eventHandlers.onPointerLeave;
    var mouseEvents = {}, touchEvents = {}, pointerEvents = {};
    if (supportsPointerEvents()) {
        pointerEvents = eventHandlers;
    }
    else if (supportsTouchEvents()) {
        touchEvents = {
            onTouchStart: onPointerDown,
            onTouchMove: onPointerMove,
            onTouchEnd: onPointerUp,
            onTouchCancel: onPointerCancel,
        };
    }
    else if (supportsMouseEvents()) {
        mouseEvents = {
            onMouseDown: onPointerDown,
            onMouseMove: onPointerMove,
            onMouseUp: onPointerUp,
            onMouseOver: onPointerOver,
            onMouseOut: onPointerOut,
            onMouseEnter: onPointerEnter,
            onMouseLeave: onPointerLeave,
        };
    }
    var pointer = useNativePointerEvents(pointerEvents, ref, options);
    var touch = useTouchEvents(touchEvents, ref, options);
    var mouse = useMouseEvents(mouseEvents, ref, options);
    return mergeUseEventResults(pointer, touch, mouse);
};
var useConditionalPointerEvents = function (eventHandlers, ref, options) {
    var pointerEventsRef;
    var pointerEventsHandlers;
    var emptyRef = React.useRef(null);
    if (ref) {
        pointerEventsRef = ref;
        pointerEventsHandlers = eventHandlers;
    }
    else {
        pointerEventsRef = emptyRef;
        pointerEventsHandlers = {};
    }
    usePointerEvents(pointerEventsHandlers, pointerEventsRef, options);
    if (ref) {
        return undefined;
    }
    else {
        return eventHandlers;
    }
};

var Point;
(function (Point) {
    Point.subtract = function (a, b) {
        return { x: a.x - b.x, y: a.y - b.y };
    };
})(Point || (Point = {}));

function startDevicePoint(session) {
    return session.pointHistory[0];
}
function lastDevicePoint(session) {
    return session.pointHistory[session.pointHistory.length - 1];
}
function getVelocity$1(session, timeDelta) {
    var pointHistory = session.pointHistory;
    if (pointHistory.length < 2) {
        return { x: 0, y: 0 };
    }
    var i = pointHistory.length - 1;
    var timestampedPoint = null;
    var lastPoint = lastDevicePoint(session);
    while (i >= 0) {
        timestampedPoint = pointHistory[i];
        if (lastPoint.timestamp - timestampedPoint.timestamp >
            timeDelta * 1000) {
            break;
        }
        i--;
    }
    if (!timestampedPoint) {
        return { x: 0, y: 0 };
    }
    var time = (lastPoint.timestamp - timestampedPoint.timestamp) / 1000;
    if (time === 0) {
        return { x: 0, y: 0 };
    }
    var currentVelocity = {
        x: (lastPoint.x - timestampedPoint.x) / time,
        y: (lastPoint.y - timestampedPoint.y) / time,
    };
    if (currentVelocity.x === Infinity) {
        currentVelocity.x = 0;
    }
    if (currentVelocity.y === Infinity) {
        currentVelocity.y = 0;
    }
    return currentVelocity;
}
function usePanGesture(_a, ref) {
    var onPan = _a.onPan, onPanStart = _a.onPanStart, onPanEnd = _a.onPanEnd;
    var session = null;
    var pointer = React.useRef(null);
    var lastMoveEvent = React.useRef(null);
    var lastMoveEventInfo = React.useRef(null);
    var updatePoint = React.useMemo(function () { return function () {
        if (!session ||
            pointer.current === null ||
            lastMoveEventInfo.current === null ||
            lastMoveEvent.current === null) {
            console.error("Pointer move without started session");
            return;
        }
        var _a = lastMoveEventInfo.current, point = _a.point, devicePoint = _a.devicePoint;
        var delta = Point.subtract(devicePoint, lastDevicePoint(session));
        var offset = Point.subtract(devicePoint, startDevicePoint(session));
        var timestamp = sync.getFrameData().timestamp;
        session.pointHistory.push(__assign({}, devicePoint, { timestamp: timestamp }));
        pointer.current.x.set(devicePoint.x);
        pointer.current.y.set(devicePoint.y);
        if (Math.abs(delta.x) > 0 || Math.abs(delta.y) > 0) {
            var velocity = getVelocity$1(session, 0.1);
            var info = {
                point: point,
                devicePoint: devicePoint,
                delta: delta,
                offset: offset,
                velocity: velocity,
            };
            if (session.startEvent) {
                if (onPan) {
                    onPan(lastMoveEvent.current, info);
                }
            }
            else {
                if (onPanStart) {
                    onPanStart(lastMoveEvent.current, info);
                }
                session.startEvent = lastMoveEvent.current;
            }
        }
        delta.x = 0;
        delta.y = 0;
    }; }, [onPan, onPanStart]);
    var onPointerMove = React.useMemo(function () { return function (event, info) {
        lastMoveEvent.current = event;
        lastMoveEventInfo.current = info;
        sync__default.update(updatePoint);
    }; }, [onPan, onPanStart]);
    var onPointerUp = React.useMemo(function () { return function (event, _a) {
        var point = _a.point, devicePoint = _a.devicePoint;
        if (!session || pointer.current === null) {
            console.error("Pointer end without started session");
            return;
        }
        sync.cancelSync.update(updatePoint);
        var delta = Point.subtract(devicePoint, lastDevicePoint(session));
        var offset = Point.subtract(devicePoint, startDevicePoint(session));
        var velocity = getVelocity$1(session, 0.1);
        stopPointerMove();
        stopPointerUp();
        if (onPanEnd) {
            onPanEnd(event, {
                point: point,
                devicePoint: devicePoint,
                delta: delta,
                offset: offset,
                velocity: velocity,
            });
        }
        session = null;
    }; }, [onPanEnd, onPointerMove]);
    var _b = usePointerEvents({ onPointerUp: onPointerUp }, window), startPointerUp = _b[0], stopPointerUp = _b[1];
    var _c = usePointerEvents({ onPointerMove: onPointerMove }, window, { capture: true }), startPointerMove = _c[0], stopPointerMove = _c[1];
    var onPointerDown = React.useMemo(function () {
        return function (event, _a) {
            var devicePoint = _a.devicePoint, point = _a.point;
            pointer.current = {
                x: motionValue(point.x),
                y: motionValue(point.y),
            };
            var timestamp = sync.getFrameData().timestamp;
            session = {
                target: event.target,
                pointHistory: [__assign({}, devicePoint, { timestamp: timestamp })],
            };
            startPointerMove();
            startPointerUp();
        };
    }, [onPointerUp, onPointerMove]);
    React.useEffect(function () {
        return function () {
            stopPointerMove();
            stopPointerUp();
        };
    }, []);
    var handlers = { onPointerDown: onPointerDown };
    if (!onPan && !onPanStart && !onPanEnd) {
        handlers = {};
    }
    return useConditionalPointerEvents(handlers, ref);
}

var order = ["hover", "tap", "drag"];
var getGesturePriority = function (gesture) {
    return order.indexOf(gesture) + 1;
};

var usePropsRef = function (props) {
    var propsRef = React.useRef(props);
    for (var key in props) {
        propsRef.current[key] = props[key];
    }
    return propsRef.current;
};
function useTapGesture(props, ref) {
    var session = null;
    var onTap = props.onTap, onTapStart = props.onTapStart, onTapCancel = props.onTapCancel, tapActive = props.tapActive, controls = props.controls;
    var propsRef = usePropsRef(props);
    var handlers = React.useMemo(function () {
        if (!onTap && !onTapStart && !onTapCancel && !tapActive) {
            return {
                onPointerUp: function () { },
                onPointerDown: function () { },
            };
        }
        var onPointerUp = function (event, _a) {
            var point = _a.point, devicePoint = _a.devicePoint;
            if (!session) {
                return;
            }
            if (controls && propsRef.tapActive) {
                controls.clearOverride(getGesturePriority("tap"));
            }
            if (!ref || event.target !== ref.current) {
                if (propsRef.onTapCancel) {
                    propsRef.onTapCancel({ point: point, devicePoint: devicePoint }, event);
                }
                return;
            }
            if (propsRef.onTap) {
                propsRef.onTap({ point: point, devicePoint: devicePoint }, event);
            }
            session = null;
        };
        var onPointerDown = function (event, _a) {
            var point = _a.point, devicePoint = _a.devicePoint;
            startPointerUp();
            if (!ref || event.target !== ref.current)
                return;
            session = {
                target: event.target,
            };
            if (propsRef.onTapStart) {
                propsRef.onTapStart({ point: point, devicePoint: devicePoint }, event);
            }
            if (controls && propsRef.tapActive) {
                controls.start(propsRef.tapActive, {
                    priority: getGesturePriority("tap"),
                });
            }
        };
        return { onPointerUp: onPointerUp, onPointerDown: onPointerDown };
    }, [
        onTap === undefined,
        onTapStart === undefined,
        onTapCancel === undefined,
    ]);
    var _a = usePointerEvents({ onPointerUp: handlers.onPointerUp }, window), startPointerUp = _a[0], stopPointerUp = _a[1];
    React.useEffect(function () { return function () {
        stopPointerUp();
    }; }, [handlers.onPointerUp]);
    return useConditionalPointerEvents({ onPointerDown: handlers.onPointerDown }, ref);
}

var hoverPriority = getGesturePriority("hover");
var useHoverGesture = function (_a, ref) {
    var hoverActive = _a.hoverActive, onHoverStart = _a.onHoverStart, onHoverEnd = _a.onHoverEnd, controls = _a.controls;
    var handlers = React.useMemo(function () {
        var onPointerEnter = function (event) {
            if (onHoverStart)
                onHoverStart(event);
            if (hoverActive && controls) {
                controls.start(hoverActive, {
                    priority: hoverPriority,
                });
            }
        };
        var onPointerLeave = function (event) {
            if (onHoverEnd)
                onHoverEnd(event);
            if (hoverActive && controls) {
                controls.clearOverride(hoverPriority);
            }
        };
        return { onPointerEnter: onPointerEnter, onPointerLeave: onPointerLeave };
    }, [hoverActive, onHoverStart, onHoverEnd, controls]);
    return useConditionalPointerEvents(handlers, ref);
};

var useGestures = function (props, ref) {
    usePanGesture(props, ref);
    useTapGesture(props, ref);
    useHoverGesture(props, ref);
};

function createLock(name) {
    var lock = null;
    return function () {
        var openLock = function () {
            lock = null;
        };
        if (lock === null) {
            lock = name;
            return openLock;
        }
        return false;
    };
}

function shouldDrag(direction, drag, currentDirection) {
    return ((drag === true || drag === "lockDirection" || drag === direction) &&
        (currentDirection === null || currentDirection === direction));
}
var getConstraints = function (axis, _a) {
    var top = _a.top, right = _a.right, bottom = _a.bottom, left = _a.left;
    if (axis === "x") {
        return { min: left, max: right };
    }
    else {
        return { min: top, max: bottom };
    }
};
var applyOverdrag = function (origin, current, dragElastic) {
    var dragFactor = typeof dragElastic === "number" ? dragElastic : 0.5;
    return popcorn.mix(origin, current, dragFactor);
};
function useDraggable(_a, ref, values, controls) {
    var _b = _a.drag, drag = _b === void 0 ? false : _b, _c = _a.dragPropagation, dragPropagation = _c === void 0 ? false : _c, dragConstraints = _a.dragConstraints, _d = _a.dragElastic, dragElastic = _d === void 0 ? true : _d, _e = _a.dragMomentum, dragMomentum = _e === void 0 ? true : _e, onDragStart = _a.onDragStart, onDragEnd = _a.onDragEnd, onDirectionLock = _a.onDirectionLock;
    var point = React.useRef({}).current;
    var origin = React.useRef({ x: 0, y: 0 }).current;
    var motionContext = React.useContext(MotionContext);
    var handlers = React.useMemo(function () {
        if (!drag)
            return {};
        var currentDirection = null;
        var openGlobalLock = null;
        if (shouldDrag("x", drag, currentDirection)) {
            point.x = values.get("x", 0);
        }
        if (shouldDrag("y", drag, currentDirection)) {
            point.y = values.get("y", 0);
        }
        var updatePoint = function (axis, offset) {
            var p = point[axis];
            if (!shouldDrag(axis, drag, currentDirection) || !p)
                return;
            var current = origin[axis] + offset[axis];
            if (dragConstraints) {
                var _a = getConstraints(axis, dragConstraints), min = _a.min, max = _a.max;
                if (min !== undefined && current < min) {
                    current = dragElastic
                        ? applyOverdrag(min, current, dragElastic)
                        : Math.max(min, current);
                }
                else if (max !== undefined && current > max) {
                    current = dragElastic
                        ? applyOverdrag(max, current, dragElastic)
                        : Math.min(max, current);
                }
            }
            p.set(current);
        };
        var onPanStart = function (event) {
            if (point.x) {
                origin.x = point.x.get();
                point.x.stop();
            }
            if (point.y) {
                origin.y = point.y.get();
                point.y.stop();
            }
            if (!dragPropagation) {
                openGlobalLock = getGlobalLock(drag);
                if (!openGlobalLock) {
                    return;
                }
            }
            currentDirection = null;
            motionContext.dragging = true;
            onDragStart && onDragStart(event);
        };
        var onPan = function (_event, _a) {
            var offset = _a.offset;
            if (!dragPropagation && !openGlobalLock) {
                return;
            }
            if (drag === "lockDirection") {
                if (currentDirection === null) {
                    currentDirection = getCurrentDirection(offset);
                    if (currentDirection !== null) {
                        onDirectionLock && onDirectionLock(currentDirection);
                    }
                    return;
                }
            }
            updatePoint("x", offset);
            updatePoint("y", offset);
        };
        var onPanEnd = function (event, _a) {
            var velocity = _a.velocity;
            if (!dragPropagation && openGlobalLock) {
                openGlobalLock();
            }
            if (dragMomentum) {
                var startMomentum = function (axis) {
                    var _a;
                    if (!shouldDrag(axis, drag, currentDirection))
                        return;
                    var transition = dragConstraints
                        ? getConstraints(axis, dragConstraints)
                        : {};
                    controls.start((_a = {},
                        _a[axis] = 0,
                        _a.transition = __assign({ type: "inertia", velocity: velocity[axis], bounceStiffness: 200, bounceDamping: 40, timeConstant: 325, restDelta: 1 }, transition),
                        _a));
                };
                startMomentum("x");
                startMomentum("y");
            }
            motionContext.dragging = false;
            onDragEnd && onDragEnd(event);
        };
        return { onPanStart: onPanStart, onPan: onPan, onPanEnd: onPanEnd };
    }, [drag, motionContext.dragging]);
    usePanGesture(handlers, ref);
}
function getCurrentDirection(offset) {
    var lockThreshold = 10;
    var direction = null;
    if (Math.abs(offset.y) > lockThreshold) {
        direction = "y";
    }
    else if (Math.abs(offset.x) > lockThreshold) {
        direction = "x";
    }
    return direction;
}
var globalHorizontalLock = createLock("dragHorizontal");
var globalVerticalLock = createLock("dragVertical");
function getGlobalLock(drag) {
    var lock = false;
    if (drag === "y") {
        lock = globalVerticalLock();
    }
    else if (drag === "x") {
        lock = globalHorizontalLock();
    }
    else {
        var openHorizontal_1 = globalHorizontalLock();
        var openVertical_1 = globalVerticalLock();
        if (openHorizontal_1 && openVertical_1) {
            lock = function () {
                openHorizontal_1();
                openVertical_1();
            };
        }
        else {
            if (openHorizontal_1)
                openHorizontal_1();
            if (openVertical_1)
                openVertical_1();
        }
    }
    return lock;
}

var useAnimateValues = function (target, controls, values, transition, onComplete) {
    var isInitialRender = React.useRef(true);
    var prevValues = React.useRef(target);
    React.useEffect(function () {
        var toAnimate = Object.keys(prevValues.current).reduce(function (acc, key) {
            var hasUpdated = target[key] !== undefined &&
                prevValues.current[key] !== target[key];
            var animateOnMount = isInitialRender.current &&
                (!values.has(key) ||
                    (values.has(key) &&
                        values.get(key).get() !==
                            target[key]));
            if (hasUpdated || animateOnMount) {
                acc[key] = target[key];
            }
            return acc;
        }, {});
        isInitialRender.current = false;
        prevValues.current = __assign({}, prevValues.current, target);
        if (Object.keys(toAnimate).length) {
            controls.start(toAnimate, transition).then(onComplete);
        }
    }, [target]);
};

var AnimatePropType;
(function (AnimatePropType) {
    AnimatePropType["Target"] = "Target";
    AnimatePropType["VariantLabel"] = "VariantLabel";
    AnimatePropType["AnimationSubscription"] = "AnimationSubscription";
})(AnimatePropType || (AnimatePropType = {}));

var _a, _b;
var makeHookComponent = function (hook) { return function (props) {
    hook(props);
    return null;
}; };
var AnimatePropComponents = (_a = {},
    _a[AnimatePropType.Target] = makeHookComponent(function (_a) {
        var animate = _a.animate, controls = _a.controls, values = _a.values, transition = _a.transition, onAnimationComplete = _a.onAnimationComplete;
        return useAnimateValues(animate, controls, values, transition, onAnimationComplete);
    }),
    _a[AnimatePropType.VariantLabel] = makeHookComponent(function (_a) {
        var animate = _a.animate, _b = _a.inherit, inherit = _b === void 0 ? true : _b, controls = _a.controls, onAnimationComplete = _a.onAnimationComplete, initial = _a.initial;
        return useVariants(animate, inherit, controls, initial, onAnimationComplete);
    }),
    _a[AnimatePropType.AnimationSubscription] = makeHookComponent(function (_a) {
        var animate = _a.animate, controls = _a.controls;
        return useAnimationSubscription(animate, controls);
    }),
    _a);
var isVariantLabel = function (prop) {
    return Array.isArray(prop) || typeof prop === "string";
};
var isAnimationSubscription = function (_a) {
    var animate = _a.animate;
    return animate instanceof AnimationManager;
};
var animatePropTypeTests = (_b = {},
    _b[AnimatePropType.Target] = function (props) {
        return props.animate !== undefined &&
            !isVariantLabel(props.animate) &&
            !isAnimationSubscription(props);
    },
    _b[AnimatePropType.VariantLabel] = function (_a) {
        var variants = _a.variants;
        return variants !== undefined;
    },
    _b[AnimatePropType.AnimationSubscription] = isAnimationSubscription,
    _b);
var getAnimatePropType = function (props) {
    for (var key in AnimatePropType) {
        if (animatePropTypeTests[key](props))
            return key;
    }
    return undefined;
};
var getAnimateComponent = function (animatePropType) {
    return animatePropType ? AnimatePropComponents[animatePropType] : undefined;
};
var gestureProps = [
    "drag",
    "onPan",
    "onPanStart",
    "onPanEnd",
    "onTap",
    "onTapStart",
    "onTapCancel",
    "tapActive",
    "onPressStart",
    "onPressEnd",
    "hoverActive",
    "onHoverStart",
    "onHoverEnd",
];
var isGesturesEnabled = function (props) {
    return gestureProps.some(function (key) { return props.hasOwnProperty(key); });
};
var isDragEnabled = function (props) { return !!props.drag; };
var Gestures = makeHookComponent(function (_a) {
    var innerRef = _a.innerRef, props = __rest(_a, ["innerRef"]);
    return useGestures(props, innerRef);
});
var Draggable = makeHookComponent(function (_a) {
    var innerRef = _a.innerRef, values = _a.values, controls = _a.controls, props = __rest(_a, ["innerRef", "values", "controls"]);
    return useDraggable(props, innerRef, values, controls);
});
var eventHandlers = new Set([
    "onTap",
    "onAnimationComplete",
    "onUpdate",
    "onHoverStart",
    "onHoverEnd",
]);
var validProps = function (props) {
    var valid = {};
    for (var key in props) {
        if (isPropValid(key) && !eventHandlers.has(key)) {
            valid[key] = props[key];
        }
    }
    return valid;
};
var RenderComponent = function (_a) {
    var base = _a.base, props = _a.props, innerRef = _a.innerRef, style = _a.style, values = _a.values;
    var isDOM = typeof base === "string";
    var isSVG = isDOM && svgElements.indexOf(base) !== -1;
    var forwardProps = isDOM ? validProps(props) : props;
    return React.createElement(base, __assign({}, forwardProps, { ref: innerRef, style: isSVG ? style : buildStyleAttr(values, style) }));
};
var checkShouldInheritVariant = function (_a) {
    var animate = _a.animate, _b = _a.inherit, inherit = _b === void 0 ? true : _b, variants = _a.variants, hoverActive = _a.hoverActive, tapActive = _a.tapActive;
    var isVariantChild = inherit && variants && !animate && !hoverActive && !tapActive;
    var isAnimationHookChild = inherit && animate instanceof AnimationManager;
    if (isVariantChild || isAnimationHookChild) {
        return true;
    }
    else {
        return false;
    }
};

var createMotionComponent = function (Component) {
    var MotionComponent = function (props, externalRef) {
        var ref = useExternalRef(externalRef);
        var values = useMotionValues(ref, props);
        var style = addMotionStyles(values, props.style);
        var animatePropType = getAnimatePropType(props);
        var shouldInheritVariant = checkShouldInheritVariant(props);
        var controls = useAnimationControls(values, props, ref, shouldInheritVariant);
        var context = useMotionContext(controls, props.initial);
        var Animate = getAnimateComponent(animatePropType);
        var handleAnimate = Animate && (React.createElement(Animate, __assign({}, props, { inherit: shouldInheritVariant, innerRef: ref, values: values, controls: controls })));
        var handleGestures = isGesturesEnabled(props) && (React.createElement(Gestures, __assign({}, props, { values: values, controls: controls, innerRef: ref })));
        var handleDrag = isDragEnabled(props) && (React.createElement(Draggable, __assign({}, props, { innerRef: ref, controls: controls, values: values })));
        var handleComponent = (React.createElement(RenderComponent, { base: Component, props: props, innerRef: ref, style: style, values: values }));
        return (React.createElement(MotionContext.Provider, { value: context },
            handleAnimate,
            handleGestures,
            handleDrag,
            handleComponent));
    };
    return React.forwardRef(MotionComponent);
};

var motion = elements.reduce(function (acc, element) {
    acc[element] = createMotionComponent(element);
    return acc;
}, {
    custom: createMotionComponent,
});

var useMotionValue = function (init) { return React.useMemo(function () { return motionValue(init); }, []); };

var useCustomValue = function (parent, getTransformer, comparitor) {
    var value = React.useRef(null);
    return React.useMemo(function () {
        if (value.current)
            value.current.destroy();
        value.current = parent.addChild({ transformer: getTransformer() });
        return value.current;
    }, comparitor);
};

var isTransformer = function (v) {
    return typeof v === "function";
};
var noop = function () { return function (v) { return v; }; };
function useTransformedValue(value, transform, to, opts) {
    var comparitor = [value];
    var transformer = noop;
    if (isTransformer(transform)) {
        transformer = function () { return transform; };
    }
    else if (Array.isArray(to)) {
        var from_1 = transform;
        transformer = function () { return popcorn.interpolate(from_1, to, opts); };
        comparitor = [value, from_1.join(","), to.join(",")];
    }
    return useCustomValue(value, transformer, comparitor);
}

var scrollX = motionValue(0);
var scrollY = motionValue(0);
var scrollXProgress = motionValue(0);
var scrollYProgress = motionValue(0);
var hasEventListener = false;
var addScrollListener = function () {
    hasEventListener = true;
    if (typeof window === "undefined")
        return;
    var updateScrollValues = function () {
        var xOffset = window.pageXOffset;
        var yOffset = window.pageYOffset;
        scrollX.set(xOffset);
        scrollY.set(yOffset);
        var maxXOffset = document.body.clientWidth - window.innerWidth;
        scrollXProgress.set(xOffset === 0 ? 0 : xOffset / maxXOffset);
        var maxYOffset = document.body.clientHeight - window.innerHeight;
        scrollYProgress.set(yOffset === 0 ? 0 : yOffset / maxYOffset);
    };
    updateScrollValues();
    window.addEventListener("resize", updateScrollValues);
    window.addEventListener("scroll", updateScrollValues, { passive: true });
};
var viewportMotionValues = {
    scrollX: scrollX,
    scrollY: scrollY,
    scrollXProgress: scrollXProgress,
    scrollYProgress: scrollYProgress,
};
var useViewportScrollValues = function () {
    if (!hasEventListener) {
        addScrollListener();
    }
    return viewportMotionValues;
};

var useAnimation = function (variants, defaultTransition) {
    var animationManager = React.useMemo(function () { return new AnimationManager(); }, []);
    if (variants) {
        animationManager.setVariants(variants);
    }
    if (defaultTransition) {
        animationManager.setDefaultTransition(defaultTransition);
    }
    React.useEffect(function () {
        animationManager.mount();
        return function () { return animationManager.unmount(); };
    }, []);
    return animationManager;
};

var useCycle = function (items, initialIndex) {
    if (initialIndex === void 0) { initialIndex = 0; }
    var numItems = items.length;
    var _a = React.useState(initialIndex), index = _a[0], setIndex = _a[1];
    return [
        items[index],
        function (next) {
            var newIndex = typeof next !== "number" ? popcorn.wrap(0, numItems, index + 1) : next;
            setIndex(newIndex);
        },
    ];
};

exports.motion = motion;
exports.useMotionValue = useMotionValue;
exports.useTransformedValue = useTransformedValue;
exports.useViewportScrollValues = useViewportScrollValues;
exports.useAnimation = useAnimation;
exports.useTapGesture = useTapGesture;
exports.usePanGesture = usePanGesture;
exports.useGestures = useGestures;
exports.useCycle = useCycle;
