import { RefObject } from "react";
import { MotionValuesMap } from "./use-motion-values";
import { TargetResolver, Transition, Variants, Target, TargetAndTransition, Variant } from "../../types";
import { MotionProps, VariantLabels } from "../types";
declare type AnimationDefinition = VariantLabels | TargetAndTransition | TargetResolver;
declare type AnimationOptions = {
    delay?: number;
    priority?: number;
};
export declare class AnimationControls<P = {}> {
    private props;
    private values;
    private ref;
    private variants;
    private baseTarget;
    private overrides;
    private resolvedOverrides;
    private defaultTransition?;
    private children?;
    private isAnimating;
    constructor(values: MotionValuesMap, ref: RefObject<Element>);
    setProps(props: P): void;
    setVariants(variants?: Variants): void;
    setDefaultTransition(transition?: Transition): void;
    setValues(target: Target, isActive?: Set<string>): void;
    checkForNewValues(target: Target): void;
    resolveVariant(variant?: Variant): {
        target?: Target;
        transition?: Transition;
        transitionEnd?: Target;
    };
    getHighestPriority(): number;
    clearOverride(overrideIndex: number): void;
    apply(definition: VariantLabels | TargetAndTransition): void;
    private applyVariantLabels;
    start(definition: AnimationDefinition, opts?: AnimationOptions): Promise<any>;
    setPriorityAnimation(definition: AnimationDefinition, priority: number): void;
    isHighestPriority(priority: number): boolean;
    private animate;
    private animateVariantLabels;
    private animateVariant;
    private animateChildren;
    private checkOverrideIsAnimating;
    private resetIsAnimating;
    stop(): void;
    addChild(controls: AnimationControls): void;
    removeChild(controls: AnimationControls): void;
    resetChildren(): void;
}
export declare const useAnimationControls: <P>(values: MotionValuesMap, props: P & MotionProps, ref: RefObject<Element>, inherit: boolean) => AnimationControls<P>;
export {};
