import { CSSProperties } from "react";
export declare type Props = {
    [key: string]: any;
};
export declare type EasingFunction = (v: number) => number;
export declare type CubicBezier = [number, number, number, number];
export declare type Easing = CubicBezier | "linear" | "easeIn" | "easeOut" | "easeInOut" | "circIn" | "circOut" | "circInOut" | "backIn" | "backOut" | "backInOut" | "anticipate" | EasingFunction;
export declare type TransitionOrchestration = {
    delay?: number;
    beforeChildren?: boolean;
    afterChildren?: boolean;
    delayChildren?: number;
    staggerChildren?: number;
    staggerDirection?: 1 | -1;
};
export declare type Tween = {
    type?: "tween";
    duration?: number;
    ease?: Easing;
    elapsed?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Spring = {
    type: "spring";
    stiffness?: number;
    damping?: number;
    mass?: number;
    restSpeed?: number;
    restDelta?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Decay = {
    type: "decay";
    modifyTarget?: (v: number) => number;
    power?: number;
    timeConstant?: number;
    restDelta?: number;
    from?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Inertia = {
    type: "inertia";
    modifyTarget?: (v: number) => number;
    bounceStiffness?: number;
    bounceDamping?: number;
    power?: number;
    timeConstant?: number;
    restDelta?: number;
    min?: number;
    max?: number;
    from?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Keyframes = {
    type: "keyframes";
    values: number[] | string[];
    easings?: Easing[];
    easeAll?: Easing;
    elapsed?: number;
    duration?: number;
    loop?: number;
    flip?: number;
    yoyo?: number;
    from?: number | string;
    to?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Physics = {
    type: "physics";
    acceleration?: number;
    friction?: number;
    restSpeed?: number | false;
    from?: number | string;
    velocity?: number;
    delay?: number;
};
export declare type Just = {
    type: "just";
    from?: number | string;
    delay?: number;
    velocity?: number;
};
export declare type None = {
    type: false;
    from?: number | string;
    delay?: number;
    velocity?: number;
};
export declare type PopmotionTransitionProps = Tween | Spring | Decay | Keyframes | Physics | Inertia | Just;
export declare type TransitionDefinition = Tween | Spring | Decay | Keyframes | Physics | Inertia | Just | None;
export declare type TransitionMap = TransitionOrchestration & {
    [key: string]: TransitionDefinition;
};
export declare type Transition = (TransitionOrchestration & TransitionDefinition) | (TransitionOrchestration & TransitionMap);
declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
declare type CSSPropertiesWithoutTransition = Omit<CSSProperties, "transition">;
export declare type Target = CSSPropertiesWithoutTransition & {
    x?: number | string;
    y?: number | string;
    z?: number | string;
    rotate?: number | string;
    rotateX?: number | string;
    rotateY?: number | string;
    rotateZ?: number | string;
    scale?: number | string;
    scaleX?: number | string;
    scaleY?: number | string;
    scaleZ?: number | string;
    skew?: number | string;
    skewX?: number | string;
    skewY?: number | string;
    originX?: number | string;
    originY?: number | string;
    originZ?: number | string;
    pathLength?: number;
    pathSpacing?: number;
};
export declare type TargetAndTransition = Target & {
    transition?: Transition;
    transitionEnd?: Target;
};
export declare type TargetResolver = (props: any, current: Target, velocity: Target) => TargetAndTransition;
export declare type Variant = TargetAndTransition | TargetResolver;
export declare type Variants = {
    [key: string]: Variant;
};
export {};
